/*
 * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
 * @license AGPL-3.0
 *
 * This code is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with OX App Suite.  If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
 *
 * Any use of the work other than as authorized under this license or copyright law is prohibited.
 *
 */

package com.openexchange.filestore.s3.internal.client;

import java.util.Map;
import com.amazonaws.Request;
import com.amazonaws.Response;
import com.amazonaws.handlers.RequestHandler2;
import com.amazonaws.services.s3.Headers;
import com.amazonaws.services.s3.model.ObjectMetadata;

/**
 * {@link ETagCorrectionHandler}
 *
 * Request handler to correct misspelled ETags in responses.
 *
 * @author <a href="mailto:tobias.friedrich@open-xchange.com">Tobias Friedrich</a>
 * @since v7.8.3
 */
public class ETagCorrectionHandler extends RequestHandler2 {

    private static final ETagCorrectionHandler INSTANCE = new ETagCorrectionHandler();

    /**
     * Gets the handler instance to correct misspelled ETag headers in responses.
     *
     * @return The handler
     */
    static ETagCorrectionHandler getInstance() {
        return INSTANCE;
    }

    /**
     * Initializes a new {@link ETagCorrectionHandler}.
     */
    private ETagCorrectionHandler() {
        super();
    }

    @Override
    public void beforeRequest(Request<?> request) {
        // nothing to do
    }

    @Override
    public void afterResponse(Request<?> request, Response<?> response) {
        Object awsResponse = response.getAwsResponse();
        if ((awsResponse instanceof ObjectMetadata)) {
            ObjectMetadata metadata = (ObjectMetadata) awsResponse;
            Map<String, Object> headers = metadata.getRawMetadata();
            if (null != headers) {
                String etag = (String) headers.get("Etag");
                if (null != etag) {
                    metadata.setHeader(Headers.ETAG, etag.replace("\"", ""));
                }
            }
        }
    }

    @Override
    public void afterError(Request<?> request, Response<?> response, Exception e) {
        // nothing to do
    }

}
